﻿using System;

namespace GeometricObjects
{
  public class Rectangle : GeometricObject, IDisposable
  {
    // ---------- Feld -------------
    private bool disposed;

    // --------- Konstruktoren ---------------
    public Rectangle() : this(0, 0, 0, 0) {}

    public Rectangle(int length, int width) : this(length, width, 0, 0) { }

    public Rectangle(int length, int width, double x, double y) 
    {
      Length = length;
      Width = width;
      _Center.X = x;
      _Center.Y = y;
      Rectangle._CountGeometricObjects++;
    }

    public Rectangle(int length, int width, Point center)
    {
      Length = length;
      Width = width;
      _Center = center;
      Rectangle._CountGeometricObjects++;
    }

    // -------- Eigenschaftsmethoden ----------
    protected int _Length;
    public virtual int Length
    {
      get { return _Length; }
      set {
        if (value >= 0)
        {
          _Length = value;
          OnPropertyChanged("Length");
        }
        else
          OnInvalidMeasure(new InvalidMeasureEventArgs(value, "Length"));
      }
    }

    protected int _Width;
    public virtual int Width
    {
        get { return _Width; }
        set
        {
          if (value >= 0)
          {
            _Width = value;
            OnPropertyChanged("Width");
          }
          else
            OnInvalidMeasure(new InvalidMeasureEventArgs(value, "Width"));
        }
    }

    // ---------- Klasseneigenschaft -----------------
    protected static int _CountRectangles;
    public static int CountRectangles
    {
        get { return _CountRectangles; }
    }

    // ---------- Instanzmethoden ----------
    public override double GetArea()
    {
        return Length * Width;
    }

    public override double GetCircumference()
    {
        return 2 * (Length + Width);
    }

    public virtual void Move(double dx, double dy, int dWidth, int dLength)
    {
      MovingEventArgs e = new MovingEventArgs();
      // Moving-Ereignis
      OnMoving(e);
      if (e.Cancel == true) return;
      XCoordinate += dx;
      YCoordinate += dy;
      Width += dWidth;
      Length += dLength;
      // Moved-Ereignis
      OnMoved(new EventArgs());
    }

    public override string ToString()
    {
      return "Rectangle, L=" + Length + ",B=" + Width + ",Fläche=" + GetArea();
    }

    // -------- Klassenmethoden ------------
    public static double GetArea(int length, int width)
    {
        return length * width;
    }   

    public static double GetCircumference(int length, int width)
    {
        return 2 * (length + width);
    }

   // Finalisierung
    public void Dispose()
    {
      if (!disposed)
      {
        Rectangle._CountRectangles--;
        GeometricObject._CountGeometricObjects--;
        GC.SuppressFinalize(this);
        disposed = true;
      }
    }

    // Destruktor
    ~Rectangle()
    {
      Dispose();
    }
  }
}
